{*****************************************************************************
 *
 *  DATreeView.pas - My TreeView
 *
 *  Copyright (c) 2000-1 Diego Amicabile
 *
 *  Author:     Diego Amicabile
 *  E-mail:     diegoami@yahoo.it
 *  Homepage:   http://www.geocities.com/diegoami
 *
 *  This component is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation;
 *
 *  This component is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this component; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *****************************************************************************}

unit uDATreeView;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls;

type
  TOnDragNodeEvent = procedure(Sender : TObject; Source, Destination : TTreeNode) of object;

  TDATreeView = class(TTreeView)
  private
    FOnDeleteNode : TTVChangedEvent;
    FCutNode : TTreeNode;
    FOnDragNodeEvent : TOnDragNodeEvent;
    FDragDropEnabled : Boolean;
    FCustomDragDrop : Boolean;
    NodDrag : TTreeNode;
  protected
    OldOnMouseDown : TMouseEvent;
    OldOnDragOver : TDragOverEvent;
    OldOnDragDrop : TDragDropEvent;
    procedure InternalMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure InternalDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
    procedure InternalDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure DeleteNodeContent(Node : TTreeNode);
  public
    procedure DeleteNode(Node : TTreeNode);
    procedure Loaded; override;

    procedure CutNode(Node : TTreeNode);
    procedure PasteNodeAt(Node : TTreeNode);
  published
    property DragDropEnabled : Boolean read FDragDropEnabled write FDragDropEnabled;
    property CustomDragDrop : Boolean read FCustomDragDrop write FCustomDragDrop;
    property OnDeleteNode : TTVChangedEvent read FOnDeleteNode write FOnDeleteNode;
    property OnDragNodeEvent : TOnDragNodeEvent read FOnDragNodeEvent write FOnDragNodeEvent;
  end;

procedure Register;

implementation

procedure TDATreeView.DeleteNode(Node : TTreeNode);
var i : integer;
  DelNode : TTreeNode;
begin
  for i := Node.Count - 1 downto 0 do begin
    DelNode := Node.Item[i];
    DeleteNode(DelNode);
  end;
  DeleteNodeContent(Node);
  Items.Delete(Node);

end;


procedure TDATreeView.DeleteNodeContent(Node : TTreeNode);
begin
  if Assigned(FOnDeleteNode) then
    FOnDeleteNode(Self,Node);
end;

procedure TDATreeView.CutNode(Node : TTreeNode);
begin
  if FCutNode <> nil then
    DeleteNode(FCutNode);
  FCutNode := Node;
end;


procedure TDATreeView.PasteNodeAt(Node : TTreeNode);

  procedure PasteNodeRoutine(FromNode, ToNode : TTreeNode);
  var i : integer;
      NewNode : TTreeNode;

  begin
    NewNode := Items.AddChildObject(ToNode, FromNode.Text, FromNode.Data);
    NewNode.ImageIndex := FromNode.ImageIndex;
    NewNode.SelectedIndex := FromNode.SelectedIndex;
    for i := 0 to FromNode.Count-1 do
      PasteNodeRoutine(FromNode.Item[i],NewNode)
  end;

begin
  PasteNodeRoutine(FCutNode,Node);
  DeleteNode(FCutNode);
  FCutNode := nil;
end;


procedure TDATreeView.InternalDragDrop(Sender, Source: TObject; X,
  Y: Integer);
var NodX : TTreeNode;
    HT : THitTests;
begin
  if Source is TTreeView then begin
    HT := GetHitTestInfoAt(X, Y);
    NodX := TTreeView(Source).GetNodeAt(X,Y);
    if (NodX <> nil) and (HT - [htOnIcon,htOnItem, htOnLabel] <> HT) then
      if (FCustomDragDrop) and (Assigned(FOnDragNodeEvent)) then
        FOnDragNodeEvent(Self,NodDrag, NodX)
      else begin
        CutNode(NodDrag);
        PasteNodeAt(Nodx);
      end;

  end;
  if Assigned(OldOnDragDrop) then
    OldOnDragDrop(Sender, Source, X, Y);
end;


procedure TDATreeView.InternalDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
var OldAccept : Boolean;
begin
  if Assigned(OldOnDragOver) then begin
    OldOnDragOver(Sender, Source, X,Y,State,OldAccept);
  end;
  Accept := OldAccept or ((Source is TTreeView) and DragDropEnabled);

end;

procedure TDATreeView.InternalMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then begin
    nodDrag := GetNodeAt(X,Y);
    if (nodDrag <> nil) and (DragDropEnabled) then
       BeginDrag(false);
  end;
  if Assigned(OldOnMouseDown) then
    OldOnMouseDown(Sender, Button, Shift, X, Y);
end;



procedure TDATreeView.Loaded;
begin
  inherited;
  OldOnDragOver := OnDragOver;
  OnDragOver := InternalDragOver;
  OldOnDragDrop := OnDragDrop;
  OnDragDrop := InternalDragDrop;
  OldOnMouseDown := OnMouseDown;
  OnMouseDown := InternalMouseDown;
end;

procedure Register;
begin
  RegisterComponents('Diego Amicabile', [TDATreeView]);
end;

end.
